@@ -67,4 +67,6 @@
 #ifdef WIN32
 #define snprintf sprintf_s
 #define strcasecmp strcmpi
+#define strtok_r strtok_s
+#define strerror_r(e, b, l) strerror_s(b, l, e)
 #endif
