@@ -76,9 +76,11 @@ int _mosquitto_packet_handle(struct mosquitto *mosq)
 int _mosquitto_handle_pingreq(struct mosquitto *mosq)
 {
 	assert(mosq);
+#ifdef WITH_STRICT_PROTOCOL
 	if(mosq->core.in_packet.remaining_length != 0){
 		return MOSQ_ERR_PROTOCOL;
 	}
+#endif
 	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PINGREQ");
 	return _mosquitto_send_pingresp(mosq);
 }
@@ -86,9 +88,11 @@ int _mosquitto_handle_pingreq(struct mosquitto *mosq)
 int _mosquitto_handle_pingresp(struct mosquitto *mosq)
 {
 	assert(mosq);
+#ifdef WITH_STRICT_PROTOCOL
 	if(mosq->core.in_packet.remaining_length != 0){
 		return MOSQ_ERR_PROTOCOL;
 	}
+#endif
 	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PINGRESP");
 	return MOSQ_ERR_SUCCESS;
 }
@@ -99,9 +103,11 @@ int _mosquitto_handle_pubackcomp(struct mosquitto *mosq, const char *type)
 	int rc;
 
 	assert(mosq);
+#ifdef WITH_STRICT_PROTOCOL
 	if(mosq->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_PROTOCOL;
 	}
+#endif
 	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
 	if(rc) return rc;
 	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received %s (Mid: %d)", type, mid);
@@ -202,9 +208,11 @@ int _mosquitto_handle_pubrec(struct mosquitto *mosq)
 	int rc;
 
 	assert(mosq);
+#ifdef WITH_STRICT_PROTOCOL
 	if(mosq->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_PROTOCOL;
 	}
+#endif
 	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
 	if(rc) return rc;
 	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PUBREC (Mid: %d)", mid);
@@ -224,9 +232,11 @@ int _mosquitto_handle_pubrel(struct mosquitto *mosq)
 	int rc;
 
 	assert(mosq);
+#ifdef WITH_STRICT_PROTOCOL
 	if(mosq->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_PROTOCOL;
 	}
+#endif
 	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
 	if(rc) return rc;
 	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PUBREL (Mid: %d)", mid);
