@@ -41,19 +41,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <memory_mosq.h>
 #include <mqtt3.h>
-
-/* DB read/write */
-const unsigned char magic[15] = {0x00, 0xB5, 0x00, 'm','o','s','q','u','i','t','t','o',' ','d','b'};
-#define DB_CHUNK_CFG 1
-#define DB_CHUNK_MSG_STORE 2
-#define DB_CHUNK_CLIENT_MSG 3
-#define DB_CHUNK_RETAIN 4
-#define DB_CHUNK_SUB 5
-/* End DB read/write */
+#include <persist.h>
 
 static int _db_restore_sub(mosquitto_db *db, const char *client_id, const char *sub, int qos);
 
-static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *client_id)
+static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *client_id, uint16_t last_mid)
 {
 	mqtt3_context *context;
 	mqtt3_context **tmp_contexts;
@@ -68,6 +60,7 @@ static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *clie
 	}
 	if(!context){
 		context = mqtt3_context_init(-1);
+		context->core.clean_session = false;
 
 		for(i=0; i<db->context_count; i++){
 			if(!db->contexts[i]){
@@ -87,6 +80,9 @@ static mqtt3_context *_db_find_or_add_context(mosquitto_db *db, const char *clie
 		}
 		context->core.id = _mosquitto_strdup(client_id);
 	}
+	if(last_mid){
+		context->core.last_mid = last_mid;
+	}
 	return context;
 }
 
@@ -102,8 +98,6 @@ static int mqtt3_db_client_messages_write(mosquitto_db *db, int db_fd, mqtt3_con
 	assert(db_fd >= 0);
 	assert(context);
 
-#define write_e(a, b, c) if(write(a, b, c) != c){ return 1; }
-
 	cmsg = context->msgs;
 	while(cmsg){
 		slen = strlen(context->core.id);
@@ -144,9 +138,10 @@ static int mqtt3_db_client_messages_write(mosquitto_db *db, int db_fd, mqtt3_con
 		cmsg = cmsg->next;
 	}
 
-#undef write_e
-
 	return MOSQ_ERR_SUCCESS;
+error:
+	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	return 1;
 }
 
 
@@ -162,8 +157,6 @@ static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
 	assert(db);
 	assert(db_fd >= 0);
 
-#define write_e(a, b, c) if(write(a, b, c) != c){ return 1; }
-
 	stored = db->msg_store;
 	while(stored){
 		length = htonl(sizeof(dbid_t) + 2+strlen(stored->source_id) +
@@ -211,15 +204,18 @@ static int mqtt3_db_message_store_write(mosquitto_db *db, int db_fd)
 		stored = stored->next;
 	}
 
-#undef write_e
-
 	return MOSQ_ERR_SUCCESS;
+error:
+	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	return 1;
 }
 
 static int mqtt3_db_client_write(mosquitto_db *db, int db_fd)
 {
 	int i;
 	mqtt3_context *context;
+	uint16_t i16temp, slen;
+	uint32_t length;
 
 	assert(db);
 	assert(db_fd >= 0);
@@ -227,11 +223,27 @@ static int mqtt3_db_client_write(mosquitto_db *db, int db_fd)
 	for(i=0; i<db->context_count; i++){
 		context = db->contexts[i];
 		if(context && context->core.clean_session == false){
+			length = htonl(2+strlen(context->core.id) + sizeof(uint16_t));
+
+			i16temp = htons(DB_CHUNK_CLIENT);
+			write_e(db_fd, &i16temp, sizeof(uint16_t));
+			write_e(db_fd, &length, sizeof(uint32_t));
+
+			slen = strlen(context->core.id);
+			i16temp = htons(slen);
+			write_e(db_fd, &i16temp, sizeof(uint16_t));
+			write_e(db_fd, context->core.id, slen);
+			i16temp = htons(context->core.last_mid);
+			write_e(db_fd, &i16temp, sizeof(uint16_t));
+
 			if(mqtt3_db_client_messages_write(db, db_fd, context)) return 1;
 		}
 	}
 
 	return MOSQ_ERR_SUCCESS;
+error:
+	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	return 1;
 }
 
 static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_subhier *node, const char *topic)
@@ -253,7 +265,6 @@ static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_
 		snprintf(thistopic, slen, "%s", node->topic);
 	}
 
-#define write_e(a, b, c) if(write(a, b, c) != c){ return 1; }
 	sub = node->subs;
 	while(sub){
 		if(sub->context->core.clean_session == false){
@@ -287,7 +298,6 @@ static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_
 		i64temp = node->retained->db_id;
 		write_e(db_fd, &i64temp, sizeof(dbid_t));
 	}
-#undef write_e
 
 	subhier = node->children;
 	while(subhier){
@@ -296,6 +306,9 @@ static int _db_subs_retain_write(mosquitto_db *db, int db_fd, struct _mosquitto_
 	}
 	_mosquitto_free(thistopic);
 	return MOSQ_ERR_SUCCESS;
+error:
+	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	return 1;
 }
 
 static int mqtt3_db_subs_retain_write(mosquitto_db *db, int db_fd)
@@ -333,7 +346,6 @@ int mqtt3_db_backup(mosquitto_db *db, bool cleanup, bool shutdown)
 		goto error;
 	}
 
-#define write_e(a, b, c) if(write(a, b, c) != c){ goto error; }
 	/* Header */
 	write_e(db_fd, magic, 15);
 	write_e(db_fd, &crc, sizeof(uint32_t));
@@ -353,7 +365,6 @@ int mqtt3_db_backup(mosquitto_db *db, bool cleanup, bool shutdown)
 	/* last db mid */
 	i64temp = db->last_db_id;
 	write_e(db_fd, &i64temp, sizeof(dbid_t));
-#undef write_e
 
 	if(mqtt3_db_message_store_write(db, db_fd)){
 		goto error;
@@ -394,6 +405,7 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 	while(store){
 		if(store->db_id == store_id){
 			cmsg->store = store;
+			cmsg->store->ref_count++;
 			break;
 		}
 		store = store->next;
@@ -403,7 +415,7 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 		mqtt3_log_printf(MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
 		return 1;
 	}
-	context = _db_find_or_add_context(db, client_id);
+	context = _db_find_or_add_context(db, client_id, 0);
 	if(!context){
 		_mosquitto_free(cmsg);
 		mqtt3_log_printf(MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
@@ -423,6 +435,44 @@ static int _db_client_msg_restore(mosquitto_db *db, const char *client_id, uint1
 	return MOSQ_ERR_SUCCESS;
 }
 
+static int _db_client_chunk_restore(mosquitto_db *db, int db_fd)
+{
+	uint16_t i16temp, slen, last_mid;
+	char *client_id = NULL;
+	int rc = 0;
+	mqtt3_context *context;
+
+	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	slen = ntohs(i16temp);
+	if(!slen){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Corrupt persistent database.");
+		close(db_fd);
+		return 1;
+	}
+	client_id = _mosquitto_calloc(slen+1, sizeof(char));
+	if(!client_id){
+		close(db_fd);
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
+	read_e(db_fd, client_id, slen);
+
+	read_e(db_fd, &i16temp, sizeof(uint16_t));
+	last_mid = ntohs(i16temp);
+
+	context = _db_find_or_add_context(db, client_id, last_mid);
+	if(!context) rc = 1;
+
+	_mosquitto_free(client_id);
+
+	return rc;
+error:
+	mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
+	if(db_fd >= 0) close(db_fd);
+	if(client_id) _mosquitto_free(client_id);
+	return 1;
+}
+
 static int _db_client_msg_chunk_restore(mosquitto_db *db, int db_fd)
 {
 	dbid_t i64temp, store_id;
@@ -431,7 +481,6 @@ static int _db_client_msg_chunk_restore(mosquitto_db *db, int db_fd)
 	char *client_id = NULL;
 	int rc;
 
-#define read_e(a, b, c) if(read(a, b, c) != c){ goto error; }
 	read_e(db_fd, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(!slen){
@@ -458,7 +507,6 @@ static int _db_client_msg_chunk_restore(mosquitto_db *db, int db_fd)
 	read_e(db_fd, &direction, sizeof(uint8_t));
 	read_e(db_fd, &state, sizeof(uint8_t));
 	read_e(db_fd, &dup, sizeof(uint8_t));
-#undef read_e
 
 	rc = _db_client_msg_restore(db, client_id, mid, qos, retain, direction, state, dup, store_id);
 	_mosquitto_free(client_id);
@@ -482,7 +530,6 @@ static int _db_msg_store_chunk_restore(mosquitto_db *db, int db_fd)
 	int rc = 0;
 	struct mosquitto_msg_store *stored = NULL;
 
-#define read_e(a, b, c) if(read(a, b, c) != c){ goto error; }
 	read_e(db_fd, &i64temp, sizeof(dbid_t));
 	store_id = i64temp;
 
@@ -556,7 +603,6 @@ static int _db_msg_store_chunk_restore(mosquitto_db *db, int db_fd)
 		}
 	}
 
-#undef read_e
 	rc = mqtt3_db_message_store(db, source_id, source_mid, topic, qos, payloadlen, payload, retain, &stored, store_id);
 	_mosquitto_free(source_id);
 	_mosquitto_free(topic);
@@ -585,7 +631,7 @@ static int _db_retain_chunk_restore(mosquitto_db *db, int db_fd)
 	store = db->msg_store;
 	while(store){
 		if(store->db_id == store_id){
-			mqtt3_sub_search(db, &db->subs, NULL, store->msg.topic, store->msg.qos, store->msg.retain, store);
+			mqtt3_db_messages_queue(db, NULL, store->msg.topic, store->msg.qos, store->msg.retain, store);
 			break;
 		}
 		store = store->next;
@@ -601,7 +647,6 @@ static int _db_sub_chunk_restore(mosquitto_db *db, int db_fd)
 	char *topic;
 	int rc = 0;
 
-#define read_e(a, b, c) if(read(a, b, c) != c){ goto error; }
 	read_e(db_fd, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	client_id = _mosquitto_calloc(slen+1, sizeof(char));
@@ -625,7 +670,6 @@ static int _db_sub_chunk_restore(mosquitto_db *db, int db_fd)
 	if(_db_restore_sub(db, client_id, topic, qos)){
 		rc = 1;
 	}
-#undef read_e
 	_mosquitto_free(client_id);
 	_mosquitto_free(topic);
 
@@ -652,7 +696,6 @@ int mqtt3_db_restore(mosquitto_db *db)
 	assert(db->config);
 	assert(db->config->persistence_filepath);
 
-#define read_e(a, b, c) if(read(a, b, c) != c){ goto error; }
 	fd = open(db->config->persistence_filepath, O_RDONLY);
 	if(fd < 0) return MOSQ_ERR_SUCCESS;
 	read_e(fd, &header, 15);
@@ -661,7 +704,10 @@ int mqtt3_db_restore(mosquitto_db *db)
 		read_e(fd, &crc, sizeof(uint32_t));
 		read_e(fd, &i32temp, sizeof(uint32_t));
 		db_version = ntohl(i32temp);
-		if(db_version != MOSQ_DB_VERSION && db_version != 0){
+		/* IMPORTANT - this is where compatibility checks are made.
+		 * Is your DB change still compatible with previous versions?
+		 */
+		if(db_version > MOSQ_DB_VERSION && db_version != 0){
 			close(fd);
 			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Unsupported persistent database format version %d (need version %d).", db_version, MOSQ_DB_VERSION);
 			return 1;
@@ -701,6 +747,10 @@ int mqtt3_db_restore(mosquitto_db *db)
 					if(_db_sub_chunk_restore(db, fd)) return 1;
 					break;
 
+				case DB_CHUNK_CLIENT:
+					if(_db_client_chunk_restore(db, fd)) return 1;
+					break;
+
 				default:
 					mqtt3_log_printf(MOSQ_LOG_WARNING, "Warning: Unsupported chunk \"%d\" in persistent database file. Ignoring.", chunk);
 					lseek(fd, length, SEEK_CUR);
@@ -712,7 +762,6 @@ int mqtt3_db_restore(mosquitto_db *db)
 		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Unable to restore persistent database. Unrecognised file format.");
 		rc = 1;
 	}
-#undef read_e
 
 	close(fd);
 
@@ -731,7 +780,7 @@ static int _db_restore_sub(mosquitto_db *db, const char *client_id, const char *
 	assert(client_id);
 	assert(sub);
 
-	context = _db_find_or_add_context(db, client_id);
+	context = _db_find_or_add_context(db, client_id, 0);
 	if(!context) return 1;
 	return mqtt3_sub_add(context, sub, qos, &db->subs);
 }
